C     Copyright (c) 2002, MAEDA Shin and NAKANO Takeo
C     All rights reserved.
C
C     Redistribution and use in source and binary forms, with or without
C     modification, are permitted provided that the following conditions
C     are met:
C
C     Redistributions of source code must retain the above copyright
C     notice, this list of conditions and the following disclaimer.
C
C     Redistributions in binary form must reproduce the above copyright
C     notice, this list of conditions and the following disclaimer in
C     the documentation and/or other materials provided with the
C     distribution.
C
C     Neither the name of the Seikei University nor the names of its
C     contributors may be used to endorse or promote products derived
C     from this software without specific prior written permission.
C
C     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
C     CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
C     INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
C     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
C     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
C     BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
C     EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
C     TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
C     DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
C     ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
C     TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
C     THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
C     SUCH DAMAGE.
C
#include "fopt.h"
      subroutine multilayer(RI, D, WL, NL, RINT, TINT)
C
C     calculate reflectance/transmittance of
C     multi layered structure with transparenet materials
C
C     List of variables
C
C     input
C     NL     :   total number of layers
C     RI(NL) :   refractive index
C     D(NL)  :   thickness
C     WL     :   wavelangth
C
C     output
C     RINT   :   Intensity reflectance
C     TINT   :   intensity transmittance
C
C     working parameters
C     C      :   light speed
C     H      :   permeabilty
C     PI     :   circular constant
C     DELTA  :   2*PI*RI(i)*D(i)/LAMBDA
C     P      :   RI/(C*H)
C     RAMP   :   amplitude reflectance
C     TAMP   :   amplitude transmittance
C
      integer NLMAX
      parameter (NLMAX=_NMAX_ + 2)

      integer NL
      double precision RI(NL), D(NL), WL
      double precision RINT, TINT

      double precision DELTA(NLMAX),P(NLMAX)

      double complex IU
      parameter (IU=(0.0d0, 1.0d0))

      double complex RAMP, TAMP, resM(2,2)
      double complex S, V, U
      double complex LagM(NLMAX,2,2), mat(2,2)

      double precision PI, C, H
      parameter (PI = 3.141592654D0)
      parameter (C = 2.99792458D8)
      parameter (H = 1.2566370614D-6)

      integer I

C
C     range check of NL
C
      if (NL .gt. NLMAX) then
         write(0, 9000) NL, NLMAX
         stop
      endif
 9000 format('(multilayer) input NL (', I4, ') > NLMAX (', I4, ')')

      do 30 I=2, NL-1
         DELTA(I) = 2 * PI * RI(I) * D(I) / WL
   30 continue

      do 40 I=1, NL
         P(I)=RI(I) / (C * H)
   40 continue

      do 50 i=2,NL-1
         LagM(i,1,1) = COS(DELTA(i))
         LagM(i,1,2) = IU * SIN(DELTA(I)) / P(i)
         LagM(i,2,1) = IU * SIN(DELTA(I)) * P(i)
         LagM(i,2,2) = LagM(i,1,1)
   50 continue

C
C           Multiplying matrices
C
      resM(1,1)=LagM(2,1,1)
      resM(1,2)=LagM(2,1,2)
      resM(2,1)=LagM(2,2,1)
      resM(2,2)=LagM(2,2,2)

      do 60 i=3,NL-1
         mat(1,1)=resM(1,1)*LagM(i,1,1)+resM(1,2)*LagM(i,2,1)
         mat(1,2)=resM(1,1)*LagM(i,1,2)+resM(1,2)*LagM(i,2,2)
         mat(2,1)=resM(2,1)*LagM(i,1,1)+resM(2,2)*LagM(i,2,1)
         mat(2,2)=resM(2,1)*LagM(i,1,2)+resM(2,2)*LagM(i,2,2)

         resM(1,1)=mat(1,1)
         resM(1,2)=mat(1,2)
         resM(2,1)=mat(2,1)
         resM(2,2)=mat(2,2)
   60 continue

C
C     calculate R and T
C
      S = resM(2,2) * P(NL) -         resM(2,1)
      V = resM(1,1)         - P(NL) * resM(1,2)
      U = P(1)*V

      RAMP = (-S + U) / (S + U)
      TAMP = 2 * P(1) / (S + U)

      RINT = ABS(RAMP)**2
      TINT = ABS(TAMP)**2 * RI(NL)/RI(1)

      return
      end
